#location-bar {
    display: none;
    position: absolute;
    z-index: 100;
    bottom: 6px;
    width: 100%;
    color: #eee;
    font-size: 150%;
    font-family: "Open Sans";
    font-weight: bold;
    text-shadow: 2px 2px 0 #000;
}

#location-bar .center {
    margin: 0 auto;
    text-align: center;
}

#location-bar .left {
    position: absolute;
    left: 10px;
    bottom: 0;
    text-align: left;
}

#location-bar .right {
    position: absolute;
    right: 10px;
    bottom: 0;
    text-align: right;
}

/*
#location-bar {
    position: absolute;
    left: 0;
    right: 130px;
    bottom: 0;
    height: 30px;
    xbackground-color: rgba(0,0,0,0.5);
    background: url('../img/location-bar.png') repeat-x;
    overflow: hidden;
}

#location-bar .container {
    position: absolute;
}

#location-bar .container > div {
    position: absolute;
    width: 32px;
    height: 30px;
    color: #eee;
    font-size: 12pt;
    line-height: 30px;
    xborder-left: 1px solid #888;
    overflow-x: no-content;
    text-align: center;
}

#location-bar .container > div:last-child {
    xborder-right: 1px solid #888;
}
*/
